﻿using NUnit.Framework;
using Moq;
using System.IO;

namespace TDD.Mocking
{
    [TestFixture]
    class OrderReaderMoqTest
    {
        private Mock<IFileReader> mockFileReader;
        private OrderReader orderReader;

        [SetUp]
        public void SetUp()
        {
            mockFileReader = new Mock<IFileReader>();
            orderReader = new OrderReader(mockFileReader.Object);
        }

        // Classic expectations approach
        [Test]
        public void TestReadOrder0()
        {
            // Use var if assigned at point of declaration: 
            // var mockFileReader = new Mock<IFileReader>();

            mockFileReader.Setup(fr => fr.ReadLine()).Returns("27412,FUBAR,876543.21").AtMostOnce();
            Order order = orderReader.ReadOrder();

            mockFileReader.VerifyAll();
            Assert.That(order.OrderId, Is.EqualTo(27412));
            Assert.That(order.Customer, Is.EqualTo("FUBAR"));
            Assert.That(order.Price, Is.EqualTo(876543.21));
        }

        [Test]
        public void TestReadOrder1()
        {
            var mockFileReader = new Mock<IFileReader>(MockBehavior.Strict);
            orderReader = new OrderReader(mockFileReader.Object);

            mockFileReader.SetupSet(fr => fr.FileName = "record.csv");
            mockFileReader.Setup(fr => fr.ReadLine()).Returns("272,ABC,543.21").AtMostOnce();
            mockFileReader.Setup(fr => fr.Close());
            Order order = orderReader.ReadOrder();

            mockFileReader.VerifySet(x => x.FileName = "record.csv");
            mockFileReader.Verify(x => x.ReadLine());
            mockFileReader.Verify(x => x.Close(), Times.Exactly(1));

            Assert.That(order.OrderId, Is.EqualTo(272));
            Assert.That(order.Customer, Is.EqualTo("ABC"));
            Assert.That(order.Price, Is.EqualTo(543.21));
        }

        [Test, ExpectedException(typeof(IOException))]
        public void TestFileReaderThrowsUp()
        {
            // make it a strict mock, but doesn't really matter
            var mockFR = new Mock<IFileReader>(MockBehavior.Strict);
            OrderReader orderReader = new OrderReader(mockFR.Object);

            mockFR.SetupSet(fr => fr.FileName = "record.csv");
            // mock the file reader implementation throwing an IOEx:
            mockFR.Setup(fr => fr.ReadLine()).Throws(new IOException("Bad file"));

            Order order = orderReader.ReadOrder();
            // Unreachable: 
            mockFR.Verify(x => x.Close(), Times.Exactly(0));
        }

    }
}
