﻿Imports NUnit.Framework
Imports Moq
Imports System.IO

Namespace TDD.Mocking
    <TestFixture()> _
    Class OrderReaderMoqTest
        Private mockFileReader As Mock(Of IFileReader)
        Private orderReader As OrderReader

        <SetUp()> _
        Public Sub SetUp()
            mockFileReader = New Mock(Of IFileReader)()
            orderReader = New OrderReader(mockFileReader.[Object])
        End Sub

        ' Classic expectations approach
        <Test()> _
        Public Sub TestReadOrder0()
            ' Use var if assigned at point of declaration: 
            ' var mockFileReader = new Mock<IFileReader>();

            mockFileReader.Setup(Function(fr) fr.ReadLine()).Returns("27412,FUBAR,876543.21").AtMostOnce()
            Dim order As Order = orderReader.ReadOrder()

            mockFileReader.VerifyAll()
            Assert.That(order.OrderId, [Is].EqualTo(27412))
            Assert.That(order.Customer, [Is].EqualTo("FUBAR"))
            Assert.That(order.Price, [Is].EqualTo(876543.21))
        End Sub

        <Test()> _
        Public Sub TestReadOrder1()
            Dim mockFileReader = New Mock(Of IFileReader)(MockBehavior.[Strict])
            orderReader = New OrderReader(mockFileReader.[Object])

            mockFileReader.SetupSet(Function(fr) InlineAssignHelper(fr.FileName, "record.csv"))
            mockFileReader.Setup(Function(fr) fr.ReadLine()).Returns("272,ABC,543.21").AtMostOnce()
            mockFileReader.Setup(Sub(fr) fr.Close())
            Dim order As Order = orderReader.ReadOrder()

            mockFileReader.VerifySet(Function(x) InlineAssignHelper(x.FileName, "record.csv"))
            mockFileReader.Verify(Function(x) x.ReadLine())
            mockFileReader.Verify(Sub(x) x.Close(), Times.Exactly(1))

            Assert.That(order.OrderId, [Is].EqualTo(272))
            Assert.That(order.Customer, [Is].EqualTo("ABC"))
            Assert.That(order.Price, [Is].EqualTo(543.21))
        End Sub

        <Test(), ExpectedException(GetType(IOException))> _
        Public Sub TestFileReaderThrowsUp()
            ' make it a strict mock, but doesn't really matter
            Dim mockFR = New Mock(Of IFileReader)(MockBehavior.[Strict])
            Dim orderReader As New OrderReader(mockFR.[Object])

            mockFR.SetupSet(Function(fr) InlineAssignHelper(fr.FileName, "record.csv"))
            ' mock the file reader implementation throwing an IOEx:
            mockFR.Setup(Function(fr) fr.ReadLine()).Throws(New IOException("Bad file"))

            Dim order As Order = orderReader.ReadOrder()
            ' Unreachable: 
            mockFR.Verify(Sub(x) x.Close(), Times.Exactly(0))
        End Sub
        Private Shared Function InlineAssignHelper(Of T)(ByRef target As T, value As T) As T
            target = value
            Return value
        End Function

    End Class
End Namespace
